/*
 * Copyright 2012 The Eventio Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.googlecode.eventio;

import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import com.googlecode.eventio.config.Configuration;
import com.googlecode.eventio.internal.EventDispatcherImpl;
import com.googlecode.eventio.internal.EventExecutorImpl;

/**
 * @author stephane.cusin
 * 
 */
public final class Eventio {

	private Eventio() {

	}

	public static EventDispatcherService newEventDispatcher() {
		return new EventDispatcherImpl();
	}

	public static EventDispatcherService newEventDispatcher(Configuration configuration) {
		return new EventDispatcherImpl(configuration);
	}

	public static EventExecutor newEventExecutor(EventDispatcher dispatcher) {
		return newEventExecutor(dispatcher, Executors.newSingleThreadExecutor());
	}

	public static EventExecutor newEventExecutor(EventDispatcher dispatcher, Executor executor) {
		return new EventExecutorImpl(dispatcher, executor);
	}

}
