/*
 * Copyright 2012 The Eventio Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.googlecode.eventio.spi;

/**
 * All this callback are executed before the action.
 * 
 * All this method are called by the unique eventio orchestrer thread.
 * 
 * @author steph
 * 
 */
public interface StatisticsImplementor {

	/**
	 * Callback about an event emitted.
	 */
	void emit(Object event);

	/**
	 * Callback about an subscription of an reaction on an event.
	 * 
	 * @param event
	 */
	void on(Object event);

	/**
	 * Callback about an subscription of an one time executed reaction.
	 * 
	 * @param event
	 */
	void once(Object event);

	/**
	 * Callback about an unsubscription reaction
	 * 
	 * @param event
	 */
	void remove(Object event);

	/**
	 * Callback about a reaction completing.
	 * 
	 * @param success
	 *            Was the reaction successful?
	 */
	public void endReaction(boolean success);
}
